/*
 * Date: Sat Apr 01 18:08:09 2000
 * Author: pIGpEN [ pigpen@s0ftpj.org, deadhead@sikurezza.org ]
 *
 * SoftProject 2000 - Digital Sekurity for Y2k
 * Sikurezza.org - Italian Security MailingList
 *
 * COFFEE-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a coffee.
 *
 * Tested on: FreeBSD 3.4-RELEASE FreeBSD 3.4-RELEASE #5: Mon Mar i386
 * 
 */

#include "ksec.h"
#include <sys/param.h>
#include <sys/proc.h>
#include <stdlib.h>


#ifdef __FreeBSD__
#include <sys/vnode.h>
#define KERNEL	/* 3.4 */
#define _KERNEL
#include <sys/file.h>
#undef KERNEL	/* 3.4 */
#undef _KERNEL

#include <sys/stat.h>
#include <sys/conf.h>


#define         BPF_CDEV        0
#define         BPF_OPEN        1
#define         BPF_CLOSE       2
#define         BPF_READ        3
#define         BPF_WRITE       4
#define         BPF_IOCTL       5
#define         BPF_POLL        6

static struct nlist bpf_list[] = {
	{       "_bpf_cdevsw"   },
	{      "_bpfopen"      	},
	{      "_bpfclose"     	},
        {       "_bpfread"      },
        {       "_bpfwrite"     },
        {       "_bpfioctl"     },
	{       "_bpfpoll"      },
        {	NULL		}
};

#define	compare(func, listindex) {				 	    \
		if (bpf_list[listindex].n_value == (u_long) b_cdevsw.func)  \
		    	printf(" ok");				            \
	        else					 	            \
			printf(" altered!");                                \
    	        putchar('\n');		 			            \
}

#endif

void
check_bpf_cdevsw(kvm_t *kd)
{
#ifdef __FreeBSD__
    struct cdevsw b_cdevsw;

    if (kvm_nlist(kd, bpf_list) == -1)
        Error_sym("[check_protosw] kvm_nlist(proto_list)");

    if (check_val_kernel(kd, BPF_CDEV) == -1)
        Error_sym("[check_bpf_cdevsw] check_val_kernel(BPF_CDEV)");

    kvm_read(kd, (u_long) bpf_list[BPF_CDEV].n_value, &b_cdevsw,
             sizeof b_cdevsw);
    printf("Checking BPF cdevsw:\n\n");

#define	c(index, s)	if (bpf_list[index].n_value == 0) 	 	       \
			 printf("%s can't be compared: unresolved symbol\n",s);\
			else				                       \

    c(BPF_OPEN, "bpfopen") {
    printf("cdevsw open function points to: %p bpfopen() is at: 0x%lx",
           b_cdevsw.d_open, bpf_list[BPF_OPEN].n_value);

    compare(d_open, BPF_OPEN);
}

c(BPF_CLOSE, "bpfclose") {
    printf("cdevsw close function points to: %p bpfclose() is at: 0x%lx",
           b_cdevsw.d_close, bpf_list[BPF_CLOSE].n_value);
    compare(d_close, BPF_CLOSE);
}

c(BPF_READ, "bpfread") {
    printf("cdevsw read function points to: %p bpfread() is at: 0x%lx",
           b_cdevsw.d_read, bpf_list[BPF_READ].n_value);

    compare(d_read, BPF_READ);
}

c(BPF_WRITE, "bpfwrite") {
    printf("cdevsw write function points to: %p bpfwrite() is at: 0x%lx",
           b_cdevsw.d_write, bpf_list[BPF_WRITE].n_value);

    compare(d_write, BPF_WRITE);
}

c(BPF_IOCTL, "bpfioctl") {
    printf("cdevsw ioctl function points to: %p bpfioctl() is at: 0x%lx",
           b_cdevsw.d_ioctl, bpf_list[BPF_IOCTL].n_value);

    compare(d_ioctl, BPF_IOCTL);
}

c(BPF_POLL, "bpfpoll") {
    printf("cdevsw poll function points to: %p bpfpoll() is at: 0x%lx",
           b_cdevsw.d_poll, bpf_list[BPF_POLL].n_value);

    compare(d_poll, BPF_POLL);
}

putchar('\n');
#else
    printf("Operation not supported\n");
#endif
}

void
bpf_to_proc(kvm_t *kd, char *name)
{
#ifdef __FreeBSD__
    /*
     * kvm_getprocs() also works..
     */

    LIST_HEAD(, proc) all;
    struct proc cur, *p;
    struct filedesc desc;
    struct file **ppfile, *pfile, file;
    int found = 0;

    if (check_val_kernel(kd, ALLPROC_N) == -1)
        Error_sym("[show_proc] check_val_kernel()");

    kvm_read(kd, list[ALLPROC_N].n_value, &all, sizeof all);

    printf("checking for proc with bpf descriptor opened ");
    printf("(use -i all to detect interface)\n\n");


    /*
     * Incomplete i've to finish it... :(
     */
    
    
    for (p = all.lh_first; p; p = cur.p_list.le_next) {

        kvm_read(kd, (u_long) p, &cur, sizeof cur);
        kvm_read(kd, (u_long) cur.p_fd, &desc, sizeof desc);

        if ((ppfile = desc.fd_ofiles) != NULL) {
            do {
                kvm_read(kd, (u_long)ppfile++, &pfile, sizeof(*pfile));

                if (!pfile)
                    continue;

                kvm_read(kd, (u_long)pfile, &file, sizeof(struct file));

                if (file.f_type == DTYPE_VNODE) {
                    struct vnode vn;

                    kvm_read(kd, (u_long)file.f_data, &vn, sizeof vn);
                    /*
                     * bpf is a character device
                     */

                    if (vn.v_type == VCHR) {
                        struct specinfo spec;

                        kvm_read(kd, (u_long)vn.v_rdev, &spec, sizeof spec);

                        /*
                         * we can also check for major = 23
                         * 
                         * ((spec.si_udev >> 8) & 0xff)==23
                         */

                        if (strstr(spec.si_name, "bpf")) {
                            if (name) {
                                if(strcmp(spec.si_name, name))
                                    continue;
                                else {
                                    found = 1;
				    break;
				}
                            } else
                                found++;

                            printf("Process with id %d (%s) has opened %s\n"
                                   , cur.p_pid, cur.p_comm, spec.si_name);
                        }
                    }
                }
            } while (pfile);
        }
    }

    if (!found) {
        if (name)
            printf("bpf descriptor not found\n");
        else
            printf("No proc with bpf descriptors opened\n");
    }

    putchar('\n');

#else
    printf("Operation not supported\n\n");
#endif
}

