/*
 * Name: if.c
 * Date: Wed Mar 08 23:39:00 2000
 * Author: pIGpEN [pigpen@s0ftpj.org, deadhead@sikurezza.org]
 *
 * SoftProject Digital Security for Y2K
 * Sikurezza.org - Italian Security Mailinglist
 *
 * COFFEE-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a coffee.
 *
 * Tested on: OpenBSD 2.6 kern#6 i386
 */


#include	"ksec.h"
#include	<sys/param.h>
#include	<sys/socket.h>
#include	<sys/ioctl.h>
#include	<net/if.h>
#include	<net/if_types.h>
#ifdef __FreeBSD__
#include	<net/if_var.h>
#endif

#include	<netinet/in.h>
#include	<arpa/inet.h>

#include	<net/bpfdesc.h>

static	void	print_if		__P((kvm_t *, struct ifnet *));
static	void	looking_if		__P((int));
static	void	print_ifstat		__P((struct if_data *));
static	void	bpf			__P((kvm_t *, struct ifnet *));

#if defined(__FreeBSD__) || defined(__bsdi__)
# define	ifhead		ifnethead
# define	next		if_link.tqe_next
#else
# define	ifhead		ifnet_head
# define	next		if_list.tqe_next
#endif

#define		read_ifh(ifh)		kvm_read(kd, list[IFNET_N].n_value,  \
					 (char *)&ifh, sizeof(struct ifhead)) 

/*
 * if_info(kd, ifpname)
 *
 * with ifpname = NULL to show all interfaces...
 * 	    	= name to show "name" interface
 * return value
 *	    = 1 if found
 *	    = 0 if not
 *
 * Note: if_info with ifp = NULL always return 1
 *
 */ 

int 
if_info(kvm_t *kd, char *ifpname) 
{
	struct ifhead ifh;
	struct ifnet  if_cur, *ifp;
#if defined(__FreeBSD__) || defined(__bsdi__)
	char name[IFNAMSIZ];
#endif

	if (check_val_kernel(kd, IFNET_N) == -1)
		Error_sym("[if_info] check_val_kernel()");

	read_ifh(ifh);

	for (ifp = ifh.tqh_first; ifp; ifp = if_cur.next) {

		bzero(&if_cur, sizeof if_cur);
		
		kvm_read(kd, (u_long) ifp, &if_cur, sizeof if_cur);

#if defined(__FreeBSD__) || defined(__bsdi__)
		bzero(name, sizeof name);
		kvm_read(kd, (u_long) if_cur.if_name, name, sizeof(name));
		snprintf(name, sizeof(name), "%s%d", name, if_cur.if_unit);

		if ((ifpname && !strcmp(ifpname,name)) || !ifpname) 
#else
		if ((ifpname && !strcmp(ifpname,if_cur.if_xname)) || !ifpname ) 
#endif
		{
			print_if(kd, &if_cur);

			if (ifpname)	
				return (1);
			else
#ifdef WAIT	
				getchar();
#else
				putchar('\n');
#endif
		}
			
	}

	return (ifpname) ? 0 : 1;

}	
 

static void 
print_if(kvm_t *kd, struct ifnet *ifp)
{
	int run = 0;
#if defined(__FreeBSD__) || defined(__bsdi__)
	char name[IFNAMSIZ];

	kvm_read(kd, (u_long) ifp->if_name, name, sizeof(name));
	printf("Interface: %s%d (internal index = %d) ", name, ifp->if_unit,
							ifp->if_index);
	snprintf(name, sizeof(name), "%s%d", name, ifp->if_unit);
	
	eth_addr(name);
#else
        printf("Interface: %s (internal index = %d) ", ifp->if_xname,                                                  		       ifp->if_index);
        eth_addr(ifp->if_xname);
#endif
	
	putchar('\n');

#define IFFBITS \
 "\020\1up\2broadcast\3debug\4loopback\5pointopoint\6smart\7running" \
 "\10noarp\11promisc\12allmulti\13oactive\14simplex\15link0\16link1\17link2" \
 "\20multicast"
		
	printf("State: ");
	printb("flags", ifp->if_flags, IFFBITS);

	putchar('\n');

	if (ifp->if_flags & IFF_UP) { 
		printf("Addr: "); 
#if defined(__FreeBSD__) || defined(__bsdi__)
		get_ifaddr(AF_INET, name, GETADDR);
#else
		get_ifaddr(AF_INET, ifp->if_xname, GETADDR);
#endif
		printf("NetMask: ");
#if defined(__FreeBSD__) || defined(__bsdi__)
		get_ifaddr(AF_INET, name, GETNMASK);
#else
		get_ifaddr(AF_INET, ifp->if_xname,GETNMASK);
#endif 
		run++;
	}

	if (ifp->if_flags & IFF_BROADCAST) {
		printf("Broadcast: ");
#if defined(__FreeBSD__) || defined(__bsdi__)
		get_ifaddr(AF_INET,name,GETBCAST);
#else
		get_ifaddr(AF_INET,ifp->if_xname,GETBCAST);
#endif
		run++;
	}

	if ((ifp->if_flags & IFF_UP) 
		&& (ifp->if_flags & IFF_POINTOPOINT)) {
		printf("Dest: ");
#if defined(__FreeBSD__) || defined(__bsdi__)
		get_ifaddr(AF_INET,name,GETDST);
#else
		get_ifaddr(AF_INET,ifp->if_xname, GETDST);
#endif
		run++;
	} 
 
	if (run)	putchar('\n');

#if defined(__FreeBSD__) || defined(__bsdi__)
	print_ifmedia(get_ifmedia(AF_INET,name));
#else
	print_ifmedia(get_ifmedia(AF_INET,ifp->if_xname));
#endif

	if (ifp->if_bpf) 
	 	bpf(kd, ifp);

	if (ifp->if_type == IFT_PPP)
#if defined(__FreeBSD__) || defined(__bsdi__)
		ppp_info(name);
#else
		ppp_info(ifp->if_xname);
#endif
	
	printf("Information for this ");
	looking_if(ifp->if_type);
	printf(" interface:\n");
	printf("\t(address length = %d, header length = %d)\n",
			ifp->if_data.ifi_addrlen, ifp->if_data.ifi_hdrlen);
	printf("\tMTU:       %lu",   ifp->if_data.ifi_mtu);
	printf("\tLinespeed: %lu\n", ifp->if_data.ifi_baudrate);

	print_ifstat(&ifp->if_data);
}


/*
 * Promisc or no Promisc ?
 */

static void
bpf(kvm_t *kd, struct ifnet *ifp)
{
	struct bpf_if ibpf;
	struct bpf_d  dbpf, *pdbpf;
	register int count = 0;
	int promisc_control = 0, no_promisc_control = 0;

 	kvm_read(kd, (u_long) ifp->if_bpf, &ibpf, sizeof ibpf);

	if (ifp->if_flags & IFF_PROMISC || ifp->if_pcount)
		printf("Expected %d promisc listener/s\n", ifp->if_pcount);

	for (pdbpf = ibpf.bif_dlist; pdbpf; pdbpf = dbpf.bd_next) {
		kvm_read(kd, (u_long) pdbpf, &dbpf, sizeof dbpf);
	
		if (dbpf.bd_promisc) {
			switch (ifp->if_pcount) {
 		 	  case 0:
		 		printf("Promisc mode is enabled "
			  	       "but there is no promisc listener!\n");
			  	break;
			  default:
			  	printf("Found a promisc listener: ");
			  	promisc_control++;
			  	break;
			}
		} else {
			printf("Found a nopromisc listener: ");
			no_promisc_control++;
		}
			
		printf("(%lu rcvd pkt, %lu drop pkt) on  #%d listener\n", 
							dbpf.bd_rcount, 
							dbpf.bd_dcount,++count);
	}
		
	if (no_promisc_control && promisc_control) {
	 printf("Warning %d nopromisc dumpers probably receive promisc packets"
			 , no_promisc_control);
	 printf(" as effect of\n another file requesting this mode...\n");
	}
	
}



static void 
print_ifstat(struct if_data *ifp)
{
	printf("\t%lu packets received, %lu packets sent\n",
					ifp->ifi_ipackets, 
					ifp->ifi_opackets);
	printf("\t%lu input errors, %lu output errors on interface\n",
					ifp->ifi_ierrors,
					ifp->ifi_oerrors);
	printf("\t%lu collisions on interface\n",
					ifp->ifi_collisions);
	printf("\t%lu octets received, %lu octets sent\n",
					ifp->ifi_ibytes,
					ifp->ifi_obytes);
	printf("\t%lu packets received, %lu packets sent via multicast\n",
					ifp->ifi_imcasts,
					ifp->ifi_omcasts); 

	printf("\t%lu dropped on input\n", 
					ifp->ifi_iqdrops);
	printf("\t%lu destined for unsupported protocols\n",
					ifp->ifi_noproto);
} 

/*
 * Type of interfaces
 */

static void 
looking_if(int type)
{
	switch (type) {
		case IFT_OTHER:		printf("Other");
					break;
		case IFT_1822:		printf("Old-style arpanet imp");
					break;
		case IFT_HDH1822:	printf("HDH arpanet imp");
					break;
		case IFT_X25DDN:	printf("X25 to imp");
					break;
		case IFT_X25:		printf("X25 (rfc887)");
					break;
		case IFT_ETHER:		printf("Ethernet");
					break;
		case IFT_ISO88023:	printf("CSMA CD");
					break;
		case IFT_ISO88024:	printf("Token bus");
					break;
		case IFT_ISO88025:	printf("Token ring");
					break;
		case IFT_ISO88026:	printf("MAN");
					break;
		case IFT_STARLAN:	printf("Starlan");
					break;
		case IFT_P10:		printf("Proteon 10Mbit ring");
					break;
		case IFT_P80:		printf("Proteon 80Mbit ring");
					break;
		case IFT_HY:		printf("Hyperchannel");
					break;
		case IFT_FDDI:		printf("FDDI");
					break;
		case IFT_LAPB:		printf("LAPB");
					break;
		case IFT_SDLC:		printf("SDLC");
					break;
		case IFT_T1:		printf("T1");
					break;
		case IFT_CEPT:		printf("European T1");
					break;
		case IFT_ISDNBASIC:	printf("ISDN Basic");
					break;
		case IFT_ISDNPRIMARY:	printf("ISDN Primary");
					break;
		case IFT_PTPSERIAL:	printf("PTP Serial");
					break;
		case IFT_PPP:		printf("PPP (rfc 1311)");
					break;
		case IFT_LOOP:		printf("Loopback");
					break;
		case IFT_EON:		printf("EON");
					break;
		case IFT_XETHER:	printf("opsolete experimental 3MB eth");
					break;
		case IFT_NSIP:		printf("XNS over IP");
					break;
		case IFT_SLIP:		printf("SLIP (IP over generic tty)");
					break;
		case IFT_ULTRA:		printf("ULTRA Technologies");
					break;
		case IFT_DS3:		printf("Generic T3");
					break;
		case IFT_SIP:		printf("SIP");
					break;
		case IFT_FRELAY:	printf("Frame Relay");
					break;
		case IFT_RS232:		printf("RS232");
					break;
		case IFT_PARA:		printf("Parallel Port");
					break;
		case IFT_ARCNET:	printf("ARCNET");
					break;
		case IFT_ARCNETPLUS:	printf("ARCNETPLUS");
					break;
		case IFT_ATM:		printf("ATM");
					break;
		case IFT_MIOX25:	printf("MIOX25");
					break;
		case IFT_SONET:		printf("SONET or SDH");
					break;
		case IFT_X25PLE:	printf("X25PLE");
					break;
		case IFT_ISO88022LLC:	printf("ISO88022LLC");
					break;
		case IFT_LOCALTALK:	printf("Local Talk");
					break;
		case IFT_SMDSDXI:	printf("SMDSDXI");
					break;
		case IFT_FRELAYDCE:	printf("Frame Relay DCE");
					break;
		case IFT_V35:		printf("V35");
					break;
		case IFT_HIPPI:		printf("HIPPI");
					break;
		case IFT_MODEM:		printf("Generic Modem");
					break;
		case IFT_AAL5:		printf("AAL5 over ATM");
					break;
		case IFT_SONETPATH:	printf("Sonetpath");
					break;
		case IFT_SONETVT:	printf("SonetVT");
					break;
		case IFT_SMDSICIP:	printf("SMDS InterCarrier");
					break;
		case IFT_PROPVIRTUAL:	printf("Proprietary Virtual/Internal");
					break;
		case IFT_PROPMUX:	printf("Proprietary Multiplexing");
					break;
#if (defined(__FreeBSD__) && (__FreeBSD_version >= 400000)) || \
    (defined(__OpenBSD__) && (OpenBSD >= 200006))
		case IFT_GIF:		printf("IPv4 & IPv6 tunneling");
					break;
		case IFT_FAITH:		printf("IPv6-to-IPv4 relaying (translation)");
					break;
#endif
										
#ifdef __OpenBSD__
		case IFT_ENC:		printf("Encapsulation");
					break;
#endif

#if defined(__OpenBSD__) && (OpenBSD >= 200006)
		case IFT_DUMMY:		printf("Dummy");
					break;

		case IFT_PVC:		printf("PVC");
					break;
#endif
		default:		printf("Unknown");
					break;
	}
}
