/*
 * Name: knstat main.c
 * Date: Fri Mar 10 17:34:41 2000
 * Author: pIGpEN [pigpen@s0ftpj.org, deadhead@sikurezza.org]
 *
 * SoftProject Digital Security for Y2K
 * Sikurezza.org - Italian Security Mailinglist
 *
 * COFFEE-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a coffee.
 *
 * Tested on: OpenBSD 2.6 kern#6 i386
 */


#include	"ksec.h"

static	void	usage	__P((int, char *));

extern	char	*optarg;


int 
main(int argn, char **arg)
{
	kvm_t *kd;
	int ch;


	if (argn == 1) {
		usage(argn, arg[0]);
		exit(1);
	}

	open_kernel(kd);

	while ((ch = getopt(argn, arg, "bB:chi:kpstT:")) != -1) {
		switch (ch) {
			case 'B':
				bpf_to_proc(kd, optarg);
				break;
			case 'b':
				bpf_to_proc(kd, NULL);
				break;
			case 'c':
				check_bpf_cdevsw(kd);
				break;
			case 'h':
				usage(argn, arg[0]);
				break;
			case 'i':	
				if(!strcmp(optarg,"all"))
					if_info(kd, NULL);
				else 
					if(!if_info(kd, optarg))  
						printf("Interface not found\n");
				break;
			case 'k':
#ifdef __FreeBSD__
				lookup_modules(kd);
#else
				printf("Not available in your kernel\n");
#endif
				break;
			case 'p':
				check_protosw(kd);
				break;
			case 's':
				check_sysent(kd);
				break;
		}
	}
	
	close_kernel(kd);

	return 0;
}

static void 
usage(int argn, char *arg)
{
	printf("--> %s\n\n", arg);
	printf("Kernel Security Checker\n\n"
	       "Usage:\n"
	       "	-B show proc with a specific bpf descriptor opened\n"
	       "	-b show proc with a bpf descriptor opened\n" 
	       "	-c check bpf cdevsw\n"
	       "	-h this help\n" 
	       "	-i [interface] show info on that interface\n"
	       "	-k show linker files\n"
	       "	-p check protocol switch (inetsw)\n"
	       "	-s check for altered syscalls\n\n");
}	
