/*
 * Name: Modules Support
 * Date: Mon Mar 27 21:21:40 2000
 * Author: pIGpEN [ pigpen@s0ftpj.org, deadhead@sikurezza.org ]
 *
 * SoftProject 2000 - Digital Sekurity for Y2k
 * Sikurezza.org - Italian Security MailingList
 *
 * COFFEE-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a coffee.
 *
 * Tested on: FreeBSD 3.4-RELEASE FreeBSD 3.4-RELEASE #4: Fri Mar i386
 */

#ifdef __FreeBSD__
#include "ksec.h"

#include <sys/param.h>
#include <sys/queue.h>
#include "linker.h"
#include <sys/module.h>


struct module {
	TAILQ_ENTRY(module) 	link;
	TAILQ_ENTRY(module) 	flink;
	struct linker_file  	*file;
	int			refs;
	int			id;
	char			*name;
	modeventhand_t		handler;
	void			*arg;
	modspecific_t		data;
};

void
lookup_modules(kvm_t *kd)
{
	struct linker_file cur;
	char name[MAXMODNAME];
	TAILQ_HEAD(, linker_file) lfh;
	struct linker_file *plf;
	struct module curmod;
	module_t pmod;
	
	bzero(&cur, sizeof cur);
	
	if (check_val_kernel(kd, MOD_N) == -1)
		Error_sym("[lookup_modules] check_val_kernel");

	kvm_read(kd, list[MOD_N].n_value, &lfh, sizeof lfh);

	printf("Kernel linker files\n\n");
	
	for (plf = lfh.tqh_first; plf; plf=cur.link.tqe_next) {
		
		bzero(name, sizeof name);
		
		kvm_read(kd, (u_long) plf, &cur, sizeof cur);
		kvm_read(kd, (u_long) cur.filename, name, sizeof name);

		printf("name: %s, id: %d, addr: %p, refs: %d, size: %x\n",
				name, cur.id, cur.address, cur.refs, cur.size);
		printf("flags: %x, userrefs: %d, dependancies: %d, file_ops at %p\n", cur.flags, cur.userrefs, cur.ndeps, cur.ops);


		printf("Modules in this file: \n");

		for (pmod = cur.modules.tqh_first; pmod; 
				pmod = curmod.flink.tqe_next) {

			char modulename[MAXMODNAME];

			bzero(modulename, sizeof modulename);
				
			kvm_read(kd, (u_long) pmod, &curmod, sizeof curmod);
			kvm_read(kd, (u_long) curmod.name, modulename,
							sizeof modulename);
	
			printf("Id=%d (%s) ", curmod.id, modulename);
			printf("mod_handler at %p refs=%d\n",
						curmod.handler, curmod.refs);

		}

		putchar('\n');
	}
		
	putchar('\n');
}
#endif
