/*
 * Name: inetsw...
 * Date: Fri Jun 16 15:13:24 2000
 * Author: pIGpEN [ pigpen@s0ftpj.org, deadhead@sikurezza.org ]
 *
 * SoftProject 2000 - Digital Sekurity for Y2k
 * Sikurezza.org - Italian Security MailingList
 *
 * COFFEE-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a coffee.
 *
 * Tested on: FreeBSD 4.0-RELEASE FreeBSD 4.0-RELEASE #4: Sun Jun i386
 *
 * Music: David Crosby, Graham Nash reissued for Atlantic's 50th Anniversary
 */

#include "ksec.h"
#include <sys/protosw.h>
#include <nlist.h>

#include <netinet/in.h>

/*
 * In FreeBSD we have to check also pointers into struct pr_usrreqs 
 * At this moment this part is not implemented :(
 */

#define EMPTY	"empty"

static struct nlist proto_list[] = {
	{	"_inetsw"	},
#define	INETSW		0
	{	"_inetdomain"	},
#define	INETDOMAIN	1
	{	"_ip_output"	},
#define	IP_OUTPUT	2
	{	"_ip_init"	},
#define	IP_INIT		3
	{	"_ip_slowtimo"	},
#define	IP_SLOWTIMO	4
	{	"_ip_drain"	},
#define	IP_DRAIN	5
	{	"_ip_sysctl"	},
#define	IP_SYSCTL	6
	{	"_ipfr_slowtimer"},
#define	IPFR_SLOWTIMO	7
	{	"_iplinit"	},
#define	IPL_INIT	8
	{	"_udp_input"	},
#define	UDP_INPUT	9
	{	"_udp_ctlinput"	},
	{	"_ip_ctloutput"},
#ifdef __FreeBSD__
	{	"_udp_usrreqs"	},
#else
	{	"_udp_usrreq"	},
#endif
	{	"_tcp_input"	},
#define TCP_INPUT	13
	{	"_tcp_ctlinput"	},
	{	"_tcp_ctloutput"},
#ifdef __FreeBSD__
	{	"_tcp_usrreqs"	},
#else
	{	"_tcp_usrreq"	},
#endif
	{	"_rip_input"	},
#define RIP_INPUT	17
#ifdef __FreeBSD__
	{	"_rip_ctlinput"	},
#else
	{	EMPTY		},
#endif
	{	"_rip_ctloutput"},
#ifdef __FreeBSD__
	{	"_rip_usrreqs"	},
#else
	{	"_rip_usrreq"	},
#endif
	{	"_icmp_input"	},
#define ICMP_INPUT	21
	{	EMPTY		},
	{	"_rip_ctloutput"},
#ifdef __FreeBSD__
	{	"_rip_usrreqs"	},
#else
	{	"_rip_usrreq"	},
#endif
	{	"_igmp_input"	},
#define IGMP_INPUT	25
	{	EMPTY		},
	{	"_rip_ctloutput"},
#ifdef __FreeBSD__
	{	"_rip_usrreqs"	},
#else
	{	"_rip_usrreq"	},
#endif		
	{	NULL		}
};

static void	check_ip	__P((kvm_t *, struct protosw *));
static void	check_proto	__P((kvm_t *, struct protosw *, int));
			
void
check_protosw(kvm_t *kd)
{
	struct protosw sw;
	int i;
	u_long offset = 0L;

	if (kvm_nlist(kd, proto_list) == -1)
		Error_sym("[check_protosw] kvm_nlist(proto_list)");

	printf("Checking integrity for inetsw:\n");

	for (i = 0; i < MAXPROTOSW; i++) {
		offset = proto_list[INETSW].n_value + ( i * sizeof sw);
		kvm_read(kd, offset, &sw, sizeof sw);

		switch (sw.pr_protocol) {
			case IPPROTO_IP:
				printf(" IP:\n");
				check_ip(kd, &sw);
				break;
			case IPPROTO_UDP:
				printf(" UDP:\n");
				check_proto(kd, &sw, UDP_INPUT);
				break;
			case IPPROTO_TCP:
				printf(" TCP:\n");
				check_proto(kd, &sw, TCP_INPUT);
				break;
			case IPPROTO_RAW:
				printf(" RAW:\n");
				check_proto(kd, &sw, RIP_INPUT);
				break;
			case IPPROTO_ICMP:
				printf(" ICMP:\n");
				check_proto(kd, &sw, ICMP_INPUT);
				break;
			case IPPROTO_IGMP:
				printf(" IGMP:\n");
				check_proto(kd, &sw, IGMP_INPUT);
				break;
		}
	}
}

static void
check_ip(kvm_t *kd, struct protosw *p)
{
	printf("     inet domain at: 0x%lx ipproto domain points to: %p", 
			proto_list[INETDOMAIN].n_value, p->pr_domain);
	
	if (proto_list[INETDOMAIN].n_value == (u_long) p->pr_domain)
		printf(" ok\n");
	else
		printf(" altered\n");

#ifdef __OpenBSD__
	printf("     ip_output at: 0x%lx ipproto output points to: %p",
			proto_list[IP_OUTPUT].n_value, p->pr_output);

	if (proto_list[IP_OUTPUT].n_value == (u_long) p->pr_output)
		printf(" ok\n");
	else
		printf(" altered\n");
#endif

	printf("     ip_init at: 0x%lx ipproto init points to: %p",
			proto_list[IP_INIT].n_value, p->pr_init);

	if (proto_list[IP_INIT].n_value == (u_long) p->pr_init)
		printf(" ok\n");
	else
		printf(" altered\n");

	if (proto_list[IPL_INIT].n_value == (u_long) p->pr_init)
		printf("     it points to iplinit (IPFilter support)\n");

	printf("     ip_slowtimo at: 0x%lx ipproto slowtimo points to: %p",
			proto_list[IP_SLOWTIMO].n_value, p->pr_slowtimo);

	if (proto_list[IP_SLOWTIMO].n_value == (u_long) p->pr_slowtimo)
		printf(" ok\n");
	else
		printf(" altered\n");
	
	if (proto_list[IPFR_SLOWTIMO].n_value == (u_long) p->pr_slowtimo)
		printf("     it points to ipfr_slowtimer (IPFilter support)\n");	

	printf("     ip_drain at: 0x%lx ipproto drain points to: %p",
			proto_list[IP_DRAIN].n_value, p->pr_drain);

	if (proto_list[IP_DRAIN].n_value == (u_long) p->pr_drain)
		printf(" ok\n");
	else
		printf(" altered\n");

#ifdef __OpenBSD__
	printf("     ip_sysctl at: 0x%lx ipproto sysctl points to: %p",
			proto_list[IP_SYSCTL].n_value, p->pr_sysctl);

	if (proto_list[IP_SYSCTL].n_value == (u_long) p->pr_sysctl)
		printf(" ok\n");
	else
		printf(" altered\n");
#endif
	
}

#define compare(str, x, y)		{				      \
	printf("\t%s:", str);						      \
	if (strcmp(proto_list[x].n_name, EMPTY)) {			      \
		if (proto_list[x].n_value == (u_long)psw->y)		      \
		 printf("ok (addr: %p)\n", psw->y);			      \
		else							      \
		 printf("altered (funct at 0x%lx protosw points to %p)\n",    \
				 proto_list[x].n_value, psw->y);	      \
	} else printf("/\n");					      	      \
	x++;								      \
}

static void 
check_proto(kvm_t *kd, struct protosw *psw, int index)
{
	int i = index;
	
	compare("pr_input", 	i, pr_input);
	compare("pr_ctlinput",	i, pr_ctlinput);
	compare("pr_ctloutput", i, pr_ctloutput);
#ifdef __FreeBSD__
	compare("pr_usrreqs",	i, pr_usrreqs);
#elif defined(__OpenBSD__)
	compare("pr_usrreq",	i, pr_usrreq);
#endif
}	
