#include "ksec.h"
#include <nlist.h>

#ifdef __FreeBSD__
# include <sys/sysent.h>
#endif

#include <sys/syscall.h>

static struct 
nlist sys_list[] = {
#define		SYSENT		0
#define         EXIT            1
#define         FORK            2
#define         READ            3
#define         WRITE           4
#define         OPEN            5
#define         CLOSE           6
#define         LINK            7
#define         UNLINK          8
#define         CHDIR           9
#define         FCHDIR          10
#define         MKNOD           11
#define         CHMOD           12
#define         CHOWN           13
#define         GETPID          14
#define         MOUNT           15
#define         UNMOUNT         16
#define         SETUID          17
#define         GETUID          18
#define         GETEUID         19
#define         IOCTL           20
#define         FCNTL           21
#define         SETSOCKOPT      22
#define         GETSOCKOPT      23
#define         GETDIRENTRIES   24
#define         GETDENTS        25
#define         SETEUID         26
#define         SETEGID         27
#define         MODSTAT         28
#define         FSTAT           29
#define         MODFIND         30
#define         GETSID          31
#define         SETSID          32
#ifdef __FreeBSD__
# define         KLDLOAD         33
# define         KLDUNLOAD       34
# define         KLDFIND         35
# define         KLDNEXT         36
# define         KLDSTAT         37
# define         KLDFIRSTMOD     38	
# define	 KLDSYM		 39
# define	 SYSCTL		 40
#endif
	{	"_sysent"	},
#ifdef __FreeBSD__
        {       "_exit"         },
        {       "_fork"         },
        {       "_read"         },
        {       "_write"        },
        {       "_open"         },
        {       "_close"        },
        {       "_link"         },
        {       "_unlink"       },
        {       "_chdir"        },
        {       "_fchdir"       },
        {       "_mknod"        },
        {       "_chmod"        },
        {       "_chown"        },
        {       "_getpid"       },       
	{       "_mount"        },
	{       "_unmount"      },
	{       "_setuid"       },
	{       "_getuid"       },
	{       "_geteuid"      },
	{       "_ioctl"        },
	{       "_fcntl"        },
	{       "_setsockopt"   },
	{       "_getsockopt"   },
        {       "_getdirentries"},
        {       "_getdents"     },
        {       "_seteuid"      },
        {       "_setegid"      },
        {       "_modstat"      },
        {       "_fstat"        },
        {       "_modfind"      },
        {       "_getsid"       },
        {       "_setsid"       },
#else
        {       "_sys_exit"     },
        {       "_sys_fork"     },
	{       "_sys_read"     },
	{       "_sys_write"    },
	{       "_sys_open"     },
	{       "_sys_close"    },
	{       "_sys_link"     },
	{       "_sys_unlink"   },
	{       "_sys_chdir"    },
	{       "_sys_fchdir"   },
	{       "_sys_mknod"    },
	{       "_sys_chmod"    },
	{       "_sys_chown"    },
	{       "_sys_getpid"   },
        {       "_sys_mount"    },
        {       "_sys_unmount"  },
        {       "_sys_setuid"   },
        {       "_sys_getuid"   },
        {       "_sys_geteuid"  },
        {       "_sys_ioctl"    },
        {       "_sys_fcntl"    },
        {       "_sys_setsockopt"},
        {       "_sys_getsockopt"},
        {       "_sys_getdirentries"},
        {       "_sys_getdents" },
        {       "_sys_seteuid"  },
        {       "_sys_setegid"  },
	{       "_sys_modstat"  },
	{       "_sys_fstat"    },
        {       "_sys_modfind"  },
	{       "_sys_getsid"   },
	{       "_sys_setsid"   },
#endif
        {       "_kldload"      },
	{       "_kldunload"    },
	{       "_kldfind"      },
	{       "_kldnext"      },
	{       "_kldstat"      },
        {       "_kldfirstmod"  },
	{	"_kldsym"	},
	{	"___sysctl"	},	
        {       NULL            }
};




#ifdef __OpenBSD__

struct proc;

struct sysent {
	short sy_narg;
	short sy_argsize;

	int	(*sy_call)	__P((struct proc *, void *, register_t *));
};
#endif


static void	check_SYS	__P((struct sysent *, int));

void
check_sysent(kvm_t *kd)
{
	struct sysent sy;
	int i;
	u_long offset = 0L;

	printf("Checking for Altered Syscall:\n\n");
	
	if (kvm_nlist(kd, sys_list) == -1)
		Error_sym("[check_sysent] check_val_kernel(SYSENT)");

	for (i = 0; i < SYS_MAXSYSCALL; i++) {
		offset = sys_list[SYSENT].n_value + ( i * sizeof sy);
		kvm_read(kd, offset, &sy, sizeof sy);
		check_SYS(&sy, i);
	}
}

#define c(index, name) {						    \
     printf("%s syscall points to %p, function is at 0x%lx : ", name,	    \
		     pent->sy_call, sys_list[index].n_value);	    	    \
	if (sys_list[index].n_value == (u_long)pent->sy_call) 		    \
		printf("ok");					    	    \
	else								    \
		printf("altered!");					    \
	putchar('\n');						            \
}									    

static void
check_SYS(struct sysent *pent, int pos)
{
	switch (pos) {
		case SYS_exit:
			c(EXIT, "exit");
			break;
		case SYS_fork:
			c(FORK, "fork");
			break;
		case SYS_read:
			c(READ, "read");
			break;
		case SYS_write:
			c(WRITE, "write");
			break;
		case SYS_open:
			c(OPEN, "open");
			break;
		case SYS_close:
			c(CLOSE, "close");
			break;
		case SYS_link:
			c(LINK, "link");
			break;
		case SYS_unlink:
			c(UNLINK, "unlink");
			break;
		case SYS_chdir:
			c(CHDIR, "chdir");
			break;
		case SYS_fchdir:
			c(FCHDIR, "fchdir");
			break;
		case SYS_mknod:
			c(MKNOD, "mknod");
			break;
		case SYS_chmod:
			c(CHMOD, "chmod");
			break;
		case SYS_chown:
			c(CHOWN, "chown");
			break;
		case SYS_getpid:
			c(GETPID, "getpid");
			break;
		case SYS_mount:
			c(MOUNT, "mount");
			break;
		case SYS_unmount:
			c(UNMOUNT, "unmount");
			break;
		case SYS_setuid:
			c(SETUID, "setuid");
			break;
		case SYS_getuid:
			c(GETUID, "getuid");
			break;
		case SYS_geteuid:
			c(GETEUID, "geteuid");
			break;
		case SYS_ioctl:
			c(IOCTL, "ioctl");
			break;
		case SYS_fcntl:
			c(FCNTL, "fcntl");
			break;
		case SYS_setsockopt:
			c(SETSOCKOPT, "setsockopt");
			break;
		case SYS_getsockopt:
			c(GETSOCKOPT, "getsockopt");
			break;
		case SYS_getdirentries:
			c(GETDIRENTRIES, "getdirentries");
			break;
		case SYS_seteuid:
			c(SETEUID, "seteuid");
			break;
		case SYS_setegid:
			c(SETEGID, "setegid");
			break;
		case SYS_fstat:
			c(FSTAT, "fstat");
			break;
		case SYS_getsid:
			c(GETSID, "getsid");
			break;
		case SYS_setsid:
			c(SETSID, "setsid");
			break;
#ifdef __FreeBSD__
		case SYS_getdents:
			c(GETDENTS, "getdents");
			break;
		case SYS___sysctl:
			c(SYSCTL, "sysctl");
			break;
		case SYS_modstat:
			c(MODSTAT, "modstat");
			break;
		case SYS_modfind:
			c(MODFIND, "modfind");
			break;
		case SYS_kldload:
			c(KLDLOAD, "kldload");
			break;
		case SYS_kldunload:
			c(KLDUNLOAD, "kldunload");
			break;
		case SYS_kldfind:
			c(KLDFIND, "kldfind");
			break;
		case SYS_kldnext:
			c(KLDNEXT, "kldnext");
			break;
		case SYS_kldstat:
			c(KLDSTAT, "kldstat");
			break;
		case SYS_kldfirstmod:
			c(KLDFIRSTMOD, "kldfirstmod");
			break;
		case SYS_kldsym:
			c(KLDSYM, "kldsym");
			break;
#endif
	}
}
