/*
 * Name: util.c
 * Date: Wed Mar 08 23:39:00 2000
 * Author: pIGpEN [pigpen@s0ftpj.org, deadhead@sikurezza.org]
 *
 * SoftProject Digital Security for Y2K
 * Sikurezza.org - Italian Security Mailinglist
 *
 * COFFEE-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a coffee.
 *
 * Tested on: OpenBSD 2.6 kern#6 i386
 */


#include	"ksec.h"
#include	<stdlib.h>
#include	<sys/param.h>
#include	<sys/ioctl.h>
#include	<sys/socket.h>
#include	<sys/sysctl.h>
#include	<net/if.h>
#include	<net/if_types.h>
#include	<net/if_media.h>
#include	<net/if_dl.h>
#include	<net/route.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include	<netdb.h>

#define		TYPE		0
#define		SUBTYPE		1


static void	ether_status	__P((struct sockaddr_dl *, int));

static const char * get_media_type_string __P((int, int));

void
get_ifaddr(int domain, char *name, int type)
{
    int s;
    struct ifreq ifr;
    struct sockaddr *sa = NULL;

    bzero(&ifr, sizeof ifr);
    strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name)-1); /* IFNAMSIZ */
    ifr.ifr_name[sizeof(ifr.ifr_name)-1] = '\0';

    if ((s = socket(domain, SOCK_DGRAM, 0)) < 0)
        Error("[get_ifaddr] socket()");

    switch (type) {
    	case GETADDR:
        	if (ioctl(s, SIOCGIFADDR, (caddr_t)&ifr) < 0)
           		Error("[get_if_addr] ioctl(SIOCGIFADDR)");
        	sa = (struct sockaddr *) &ifr.ifr_addr;
        	break;
    	case GETBCAST:
        	if (ioctl(s, SIOCGIFBRDADDR, (caddr_t)&ifr) < 0)
          		Error("[get_if_addr] ioctl(SIOCGIFBRDADDR)");
        	sa = (struct sockaddr *) &ifr.ifr_broadaddr;
        	break;
   	case GETDST:
        	if (ioctl(s, SIOCGIFDSTADDR, (caddr_t)&ifr) < 0)
            		Error("[get_if_addr] ioctl(SIOCGIFDSTADDR)");
        	sa = (struct sockaddr *) &ifr.ifr_dstaddr;
        	break;
	case GETNMASK:
        	if (ioctl(s, SIOCGIFNETMASK, (caddr_t)&ifr) < 0)
            		Error("[get_if_addr] ioctl(SIOCGIFNETMASK)");
        	sa = (struct sockaddr *) &ifr.ifr_addr;
        	break;
    }

    if (type == GETNMASK) {
        register struct sockaddr_in *si;

        si = (struct sockaddr_in *) sa;
        printf("0x%x ", (u_int) ntohl(si->sin_addr.s_addr));
    } else
        p_addr(sa, 0);

    close(s);
}


struct ifmediareq *
            get_ifmedia(int domain, char *name)
{
    static struct ifmediareq ifrm;
    int s, error = 0;

    bzero(&ifrm, sizeof ifrm);
    strncpy(ifrm.ifm_name, name, sizeof(ifrm.ifm_name)-1);
    ifrm.ifm_name[sizeof(ifrm.ifm_name)-1] = '\0';

    if ((s = socket(domain, SOCK_DGRAM, 0)) < 0)
        Error("[get_ifmedia] socket");

    if (ioctl(s, SIOCGIFMEDIA, (caddr_t) &ifrm) < 0)
        error = 1;

    close(s);

    return (!error) ? &ifrm :  0 ;
}

static struct ifmedia_description ifm_type_descr[] = IFM_TYPE_DESCRIPTIONS;
#ifndef __FreeBSD__
static struct ifmedia_description ifm_subs_descr[] = IFM_SUBTYPE_DESCRIPTIONS;
#else
static struct ifmedia_description ifm_subs_eth[] =
                IFM_SUBTYPE_ETHERNET_DESCRIPTIONS;

static struct ifmedia_description ifm_subs_tok[] =
                IFM_SUBTYPE_TOKENRING_DESCRIPTIONS;

static struct ifmedia_description ifm_subs_fddi[] =
                IFM_SUBTYPE_FDDI_DESCRIPTIONS;

static struct ifmedia_description ifm_def[] =
                IFM_SUBTYPE_SHARED_DESCRIPTIONS;
#endif

/*
 * Incomplete ...
 */

int
print_ifmedia(struct ifmediareq *ifm)
{
    if (!ifm)	return 0;

    printf("%s %s", 	get_media_type_string(ifm->ifm_current, TYPE) ,
           		get_media_type_string(ifm->ifm_current, SUBTYPE));

    printf(" State= ");
    switch (ifm->ifm_current) {
    	case IFM_ETHER:
        	(ifm->ifm_status & IFM_ACTIVE)? printf("active") :
        					printf("no carrier");
        	break;
    	case IFM_FDDI:
   	case IFM_TOKEN:
        	(ifm->ifm_status & IFM_ACTIVE)?	printf("active") :
        					printf("no ring");
        	break;
    	default:
        	printf("unknown");
    }

    putchar('\n');
    return 1;
}


#ifdef __FreeBSD__

#define IFM_TYPE_MATCH(dt, t)		\
	(IFM_TYPE((dt)) == 0 || IFM_TYPE((dt)) == IFM_TYPE((t)))

#endif 


static const char *
get_media_type_string(int mword, int type)
{
    static struct ifmedia_description *desc = NULL;

    switch (type) {
	    case TYPE:
        	for (desc=ifm_type_descr; desc->ifmt_string != NULL; desc++) {
            		if ( IFM_TYPE(mword) == desc->ifmt_word )
                		return (desc->ifmt_string);
        	}
        	break;

    	    case SUBTYPE:
#ifndef __FreeBSD__
        	for (desc=ifm_subs_descr; desc->ifmt_string != NULL; desc++)
#else
	        switch (mword) {
        		case IFM_ETHER:
            			desc = ifm_subs_eth;
            			break;
        		case IFM_TOKEN:
            			desc = ifm_subs_tok;
           			break;
       			case IFM_FDDI:
            			desc = ifm_subs_fddi;
            			break;
		        default:
				desc = ifm_def;
		            	break;
        	}

        	for (;desc->ifmt_string != NULL; desc++) 
#endif			
		{
            		if ( IFM_TYPE_MATCH(desc->ifmt_word, mword) &&
                    	     IFM_SUBTYPE(desc->ifmt_word) == IFM_SUBTYPE(mword))
		        	return (desc->ifmt_string);
        	}
        	break;
    	    default:
        	Error("[get_media_type_string] checking type");
    }

    return NULL;
}

void
p_addr(struct sockaddr *sa, int type)
{
    switch (sa->sa_family) {
	    case AF_INET:
            {
             struct sockaddr_in *si = (struct sockaddr_in *) sa;

             if (si->sin_addr.s_addr != INADDR_ANY)
               	printf("%s ", inet_ntoa(si->sin_addr));
             else
               	printf("default");
             if (type && si->sin_addr.s_addr != INADDR_ANY)
               	printf("(%s)", addr_to_name(AF_INET, si->sin_addr.s_addr));
	    }
            break;
	    
	   case AF_LINK:
	   {
            struct sockaddr_dl *sdl = (struct sockaddr_dl *) sa;

            if (!sdl->sdl_nlen && !sdl->sdl_alen
                    && !sdl->sdl_slen) {
                printf("link#%d", sdl->sdl_index);
            } else {
                switch (sdl->sdl_type) {
                	case IFT_ETHER:
                    		ether_status(sdl, type);
                    		break;
                	default:
                    		printf("%s", link_ntoa(sdl));
                    		break;
                }
            }
        }
    }
}



/*
 * This function gives you MAC Addresses...
 * Obtained via sysctl()
 */


void
eth_addr(char *ifp)
{
    char *buff, *lim, *next;
    int mib[6];
    size_t needed;
    struct if_msghdr *ifm;
    struct sockaddr_dl *sdl;
    char name[IFNAMSIZ+1];

    mib[0] = CTL_NET;
    mib[1] = PF_ROUTE;
    mib[2] = 0;
    mib[3] = 0;
    mib[4] = NET_RT_IFLIST;
    mib[5] = 0;

    if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0)
        Error("[eth_addr] sysctl #1");

    if ((buff=malloc(needed)) == NULL)
        Error("[eth_addr] malloc");

    if (sysctl(mib, 6, buff, &needed, NULL, 0) < 0)
        Error("[eth_addr] sysctl #2");

    lim = buff + needed;

    next = buff;

    while (next < lim) {
        ifm = (struct if_msghdr *) next;
        if (ifm->ifm_type == RTM_IFINFO) {
            sdl = (struct sockaddr_dl *)(ifm + 1);

            strncpy(name, sdl->sdl_data, sizeof(name)-1);
            name[sizeof(name)-1] = '\0';

            if (!strcmp(name,ifp))
                ether_status(sdl, 1);
        }

        next += ifm->ifm_msglen;
    }

    free(buff);
}

static void
ether_status(struct sockaddr_dl *sdl, int type)
{
    char *cp;
    int n;

    cp = (char *)LLADDR(sdl);
    if ((n = sdl->sdl_alen) > 0) {
        if (type) {
            if (sdl->sdl_type == IFT_ETHER)
                printf("Ether ");
            else
                printf("lladdr ");
        }
        while (--n >= 0)
            printf("%02x%c", *cp++ & 0xff, n>0? ':' : ' ');
    }
}


char *
addr_to_name(int domain, u_long in)
{
    struct hostent *hp;
    static char line[MAXHOSTNAMELEN + 1];
    char *cp = NULL;

    if ((hp = gethostbyaddr((char *)&in, sizeof(struct in_addr), domain)))
        cp = hp->h_name;

    if (cp) {
        strncpy(line, cp, sizeof(line)-1);
        line[sizeof(line)-1] = '\0';
    } else {
#define C(x) 	((x) & 0xff)
        in = ntohl(in);
        snprintf(line, sizeof(line), "%lu.%lu.%lu.%lu",
                 C(in >> 24), C(in >> 16), C(in >> 8), C(in));
    }

    return (line);
}

/*
 * Print a value a la the %b format of the kernel's printf
 */
void
printb(s, v, bits)
const char *s;
register unsigned v;
register const char *bits;
{
    register int i, any = 0;
    register char c;

    bits++;
    if (bits) {
        putchar('<');
        while ((i = *bits++) != '\0') {
            if (v & (1 << (i-1))) {
                if (any)
                    putchar(',');
                any = 1;
                for (; (c = *bits) > 32; bits++)
                    putchar(c);
            } else
                for (; *bits > 32; bits++)
                    ;
        }
        putchar('>');
    }
}
